@@ -29,6 +29,7 @@
 #include "cogl-display.h"
 #include "cogl-renderer.h"
 #include "cogl-onscreen-template.h"
+#include "cogl-internal.h"
 #if COGL_HAS_WAYLAND_EGL_SERVER_SUPPORT
 #include <wayland-server.h>
 #endif
@@ -46,7 +47,7 @@ struct _CoglDisplay
 #endif
 
 #ifdef COGL_HAS_EGL_PLATFORM_GDL_SUPPORT
-  struct gdl_plane *gdl_plane;
+  gdl_plane_id_t gdl_plane;
 #endif
 
   void *winsys;
