@@ -38,6 +38,7 @@
 #include "cogl-profile.h"
 #include "cogl-attribute-private.h"
 #include "cogl-point-in-poly-private.h"
+#include "cogl-private.h"
 
 #include <string.h>
 #include <gmodule.h>
@@ -95,6 +96,8 @@ typedef struct _CoglJournalFlushState
 {
   CoglJournal         *journal;
 
+  CoglFramebuffer     *framebuffer;
+
   CoglAttributeBuffer *attribute_buffer;
   GArray              *attributes;
   int                  current_attribute;
@@ -102,10 +105,10 @@ typedef struct _CoglJournalFlushState
   gsize                stride;
   size_t               array_offset;
   GLuint               current_vertex;
-#ifndef HAVE_COGL_GL
+
   CoglIndices         *indices;
   gsize                indices_type_size;
-#endif
+
   CoglMatrixStack     *modelview_stack;
   CoglMatrixStack     *projection_stack;
 
@@ -270,8 +273,7 @@ _cogl_journal_flush_modelview_and_entries (CoglJournalEntry *batch_start,
   CoglAttribute **attributes;
   CoglDrawFlags draw_flags = (COGL_DRAW_SKIP_JOURNAL_FLUSH |
                               COGL_DRAW_SKIP_PIPELINE_VALIDATION |
-                              COGL_DRAW_SKIP_FRAMEBUFFER_FLUSH |
-                              COGL_DRAW_SKIP_LEGACY_STATE);
+                              COGL_DRAW_SKIP_FRAMEBUFFER_FLUSH);
 
   COGL_STATIC_TIMER (time_flush_modelview_and_entries,
                      "flush: pipeline+entries", /* parent */
@@ -295,41 +297,44 @@ _cogl_journal_flush_modelview_and_entries (CoglJournalEntry *batch_start,
     }
 
   attributes = (CoglAttribute **)state->attributes->data;
-  cogl_push_source (state->source);
+  _cogl_push_source (state->source, FALSE);
 
   if (!_cogl_pipeline_get_real_blend_enabled (state->source))
     draw_flags |= COGL_DRAW_COLOR_ATTRIBUTE_IS_OPAQUE;
 
 #ifdef HAVE_COGL_GL
-
-  /* XXX: it's rather evil that we sneak in the GL_QUADS enum here... */
-  _cogl_draw_attributes (GL_QUADS,
-                         state->current_vertex, batch_len * 4,
-                         attributes,
-                         state->attributes->len,
-                         draw_flags);
-
-#else /* HAVE_COGL_GL */
-  if (batch_len > 1)
+  if (ctx->driver == COGL_DRIVER_GL)
     {
-      _cogl_draw_indexed_attributes (COGL_VERTICES_MODE_TRIANGLES,
-                                     state->current_vertex * 6 / 4,
-                                     batch_len * 6,
-                                     state->indices,
-                                     attributes,
-                                     state->attributes->len,
-                                     draw_flags);
-
-    }
-  else
-    {
-      _cogl_draw_attributes (COGL_VERTICES_MODE_TRIANGLE_FAN,
-                             state->current_vertex, 4,
+      /* XXX: it's rather evil that we sneak in the GL_QUADS enum here... */
+      _cogl_draw_attributes (GL_QUADS,
+                             state->current_vertex, batch_len * 4,
                              attributes,
                              state->attributes->len,
                              draw_flags);
     }
-#endif
+  else
+#endif /* HAVE_COGL_GL */
+    {
+      if (batch_len > 1)
+        {
+          _cogl_draw_indexed_attributes (COGL_VERTICES_MODE_TRIANGLES,
+                                         state->current_vertex * 6 / 4,
+                                         batch_len * 6,
+                                         state->indices,
+                                         attributes,
+                                         state->attributes->len,
+                                         draw_flags);
+
+        }
+      else
+        {
+          _cogl_draw_attributes (COGL_VERTICES_MODE_TRIANGLE_FAN,
+                                 state->current_vertex, 4,
+                                 attributes,
+                                 state->attributes->len,
+                                 draw_flags);
+        }
+    }
 
   /* DEBUGGING CODE XXX: This path will cause all rectangles to be
    * drawn with a coloured outline. Each batch will be rendered with
@@ -619,9 +624,8 @@ _cogl_journal_flush_vbo_offsets_and_entries (CoglJournalEntry *batch_start,
                         4,
                         COGL_ATTRIBUTE_TYPE_UNSIGNED_BYTE);
 
-#ifndef HAVE_COGL_GL
-  state->indices = cogl_get_rectangle_indices (batch_len);
-#endif
+  if (ctx->driver != COGL_DRIVER_GL)
+    state->indices = cogl_get_rectangle_indices (batch_len);
 
   /* We only create new Attributes when the stride within the
    * AttributeBuffer changes. (due to a change in the number of pipeline
@@ -703,7 +707,7 @@ _cogl_journal_flush_clip_stacks_and_entries (CoglJournalEntry *batch_start,
   if (G_UNLIKELY (COGL_DEBUG_ENABLED (COGL_DEBUG_BATCHING)))
     g_print ("BATCHING:  clip stack batch len = %d\n", batch_len);
 
-  _cogl_clip_stack_flush (batch_start->clip_stack);
+  _cogl_clip_stack_flush (batch_start->clip_stack, state->framebuffer);
 
   _cogl_matrix_stack_push (state->modelview_stack);
 
@@ -904,6 +908,10 @@ can_software_clip_entry (CoglJournalEntry *journal_entry,
       clip_bounds_out->y_2 = MIN (clip_bounds_out->y_2, rect_y2 - ty);
     }
 
+  if (clip_bounds_out->x_2 <= clip_bounds_out->x_1 ||
+      clip_bounds_out->y_2 <= clip_bounds_out->y_1)
+    memset (clip_bounds_out, 0, sizeof (ClipBounds));
+
   return TRUE;
 }
 
@@ -1350,6 +1358,7 @@ _cogl_journal_flush (CoglJournal *journal,
    * that the timer isn't started recursively. */
   COGL_TIMER_START (_cogl_uprof_context, flush_timer);
 
+  state.framebuffer = framebuffer;
   cogl_push_framebuffer (framebuffer);
 
   if (G_UNLIKELY (COGL_DEBUG_ENABLED (COGL_DEBUG_BATCHING)))
@@ -1536,12 +1545,6 @@ _cogl_journal_log_quad (CoglJournal  *journal,
 
   source = pipeline;
 
-  if (G_UNLIKELY (ctx->legacy_state_set))
-    {
-      source = cogl_pipeline_copy (pipeline);
-      _cogl_pipeline_apply_legacy_state (source);
-    }
-
   flush_options.flags = 0;
   if (G_UNLIKELY (cogl_pipeline_get_n_layers (pipeline) != n_layers))
     {
@@ -1558,9 +1561,7 @@ _cogl_journal_log_quad (CoglJournal  *journal,
 
   if (G_UNLIKELY (flush_options.flags))
     {
-      /* If we haven't already created a derived pipeline... */
-      if (source == pipeline)
-        source = cogl_pipeline_copy (pipeline);
+      source = cogl_pipeline_copy (pipeline);
       _cogl_pipeline_apply_overrides (source, &flush_options);
     }
 
@@ -1647,8 +1648,7 @@ entry_to_screen_polygon (const CoglJournalEntry *entry,
                               poly, /* points_out */
                               4 /* n_points */);
 
-  _cogl_framebuffer_get_viewport4fv (cogl_get_draw_framebuffer (),
-                                     viewport);
+  cogl_framebuffer_get_viewport4fv (cogl_get_draw_framebuffer (), viewport);
 
 /* Scale from OpenGL normalized device coordinates (ranging from -1 to 1)
  * to Cogl window/framebuffer coordinates (ranging from 0 to buffer-size) with
