@@ -239,8 +239,6 @@ _cogl_pipeline_fragend_arbfp_start (CoglPipeline *pipeline,
       shader_state->user_program = user_program;
       if (user_program == COGL_INVALID_HANDLE)
         {
-          int i;
-
           /* We reuse a single grow-only GString for code-gen */
           g_string_set_size (ctx->codegen_source_buffer, 0);
           shader_state->source = ctx->codegen_source_buffer;
@@ -389,7 +387,10 @@ setup_arg (CoglPipeline *pipeline,
       arg->name = "texel%d";
       arg->texture_unit = _cogl_pipeline_layer_get_unit_index (layer);
       texture = _cogl_pipeline_layer_get_texture (layer);
-      cogl_texture_get_gl_texture (texture, NULL, &gl_target);
+      if (texture)
+        cogl_texture_get_gl_texture (texture, NULL, &gl_target);
+      else
+        gl_target = GL_TEXTURE_2D;
       setup_texture_source (shader_state, arg->texture_unit, gl_target);
       break;
     case COGL_PIPELINE_COMBINE_SOURCE_CONSTANT:
@@ -422,7 +423,10 @@ setup_arg (CoglPipeline *pipeline,
       arg->name = "texture[%d]";
       arg->texture_unit = src - GL_TEXTURE0;
       texture = _cogl_pipeline_layer_get_texture (layer);
-      cogl_texture_get_gl_texture (texture, NULL, &gl_target);
+      if (texture)
+        cogl_texture_get_gl_texture (texture, NULL, &gl_target);
+      else
+        gl_target = GL_TEXTURE_2D;
       setup_texture_source (shader_state, arg->texture_unit, gl_target);
     }
 
