@@ -24,14 +24,17 @@
 #ifndef __COGL_RENDERER_PRIVATE_H
 #define __COGL_RENDERER_PRIVATE_H
 
+#include <gmodule.h>
+
 #include "cogl-object-private.h"
 #include "cogl-winsys-private.h"
+#include "cogl-internal.h"
 
 #ifdef COGL_HAS_XLIB_SUPPORT
 #include <X11/Xlib.h>
 #endif
 
-#if COGL_HAS_EGL_PLATFORM_WAYLAND_SUPPORT
+#if defined (COGL_HAS_EGL_PLATFORM_WAYLAND_SUPPORT)
 #include <wayland-client.h>
 #endif
 
@@ -44,7 +47,13 @@ struct _CoglRenderer
 #ifdef COGL_HAS_XLIB_SUPPORT
   Display *foreign_xdpy;
 #endif
-#if COGL_HAS_EGL_PLATFORM_WAYLAND_SUPPORT
+
+  CoglDriver driver;
+#ifndef HAVE_DIRECTLY_LINKED_GL_LIBRARY
+  GModule *libgl_module;
+#endif
+
+#if defined (COGL_HAS_EGL_PLATFORM_WAYLAND_SUPPORT)
   struct wl_display *foreign_wayland_display;
   struct wl_compositor *foreign_wayland_compositor;
 #endif
@@ -70,4 +79,8 @@ _cogl_renderer_remove_native_filter (CoglRenderer *renderer,
                                      CoglNativeFilterFunc func,
                                      void *data);
 
+void *
+_cogl_renderer_get_proc_address (CoglRenderer *renderer,
+                                 const char *name);
+
 #endif /* __COGL_RENDERER_PRIVATE_H */
