@@ -136,6 +136,8 @@ _cogl_texture_2d_set_wrap_mode_parameters (CoglTexture *tex,
 {
   CoglTexture2D *tex_2d = COGL_TEXTURE_2D (tex);
 
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
   /* Only set the wrap mode if it's different from the current value
      to avoid too many GL calls. Texture 2D doesn't make use of the r
      coordinate so we can ignore its wrap mode */
@@ -145,8 +147,12 @@ _cogl_texture_2d_set_wrap_mode_parameters (CoglTexture *tex,
       _cogl_bind_gl_texture_transient (GL_TEXTURE_2D,
                                        tex_2d->gl_texture,
                                        tex_2d->is_foreign);
-      GE( glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrap_mode_s) );
-      GE( glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrap_mode_t) );
+      GE( ctx, glTexParameteri (GL_TEXTURE_2D,
+                                GL_TEXTURE_WRAP_S,
+                                wrap_mode_s) );
+      GE( ctx, glTexParameteri (GL_TEXTURE_2D,
+                                GL_TEXTURE_WRAP_T,
+                                wrap_mode_t) );
 
       tex_2d->wrap_mode_s = wrap_mode_s;
       tex_2d->wrap_mode_t = wrap_mode_t;
@@ -171,6 +177,8 @@ _cogl_texture_2d_can_create (unsigned int width,
   GLenum gl_intformat;
   GLenum gl_type;
 
+  _COGL_GET_CONTEXT (ctx, FALSE);
+
   /* If NPOT textures aren't supported then the size must be a power
      of two */
   if (!cogl_features_available (COGL_FEATURE_TEXTURE_NPOT) &&
@@ -178,13 +186,13 @@ _cogl_texture_2d_can_create (unsigned int width,
        !_cogl_util_is_pot (height)))
     return FALSE;
 
-  _cogl_pixel_format_to_gl (internal_format,
-                            &gl_intformat,
-                            NULL,
-                            &gl_type);
+  ctx->texture_driver->pixel_format_to_gl (internal_format,
+                                           &gl_intformat,
+                                           NULL,
+                                           &gl_type);
 
   /* Check that the driver can create a texture with that size */
-  if (!_cogl_texture_driver_size_supported (GL_TEXTURE_2D,
+  if (!ctx->texture_driver->size_supported (GL_TEXTURE_2D,
                                             gl_intformat,
                                             gl_type,
                                             width,
@@ -250,20 +258,20 @@ cogl_texture_2d_new_with_size (CoglContext *ctx,
       return NULL;
     }
 
-  internal_format = _cogl_pixel_format_to_gl (internal_format,
-                                              &gl_intformat,
-                                              &gl_format,
-                                              &gl_type);
+  internal_format = ctx->texture_driver->pixel_format_to_gl (internal_format,
+                                                             &gl_intformat,
+                                                             &gl_format,
+                                                             &gl_type);
 
   tex_2d = _cogl_texture_2d_create_base (width, height, COGL_TEXTURE_NONE,
                                          internal_format);
 
-  _cogl_texture_driver_gen (GL_TEXTURE_2D, 1, &tex_2d->gl_texture);
+  ctx->texture_driver->gen (GL_TEXTURE_2D, 1, &tex_2d->gl_texture);
   _cogl_bind_gl_texture_transient (GL_TEXTURE_2D,
                                    tex_2d->gl_texture,
                                    tex_2d->is_foreign);
-  GE( glTexImage2D (GL_TEXTURE_2D, 0, gl_intformat,
-                    width, height, 0, gl_format, gl_type, NULL) );
+  GE( ctx, glTexImage2D (GL_TEXTURE_2D, 0, gl_intformat,
+                         width, height, 0, gl_format, gl_type, NULL) );
 
   return _cogl_texture_2d_handle_new (tex_2d);
 }
@@ -281,6 +289,8 @@ _cogl_texture_2d_new_from_bitmap (CoglBitmap      *bmp,
   GLenum         gl_type;
   guint8        *data;
 
+  _COGL_GET_CONTEXT (ctx, COGL_INVALID_HANDLE);
+
   g_return_val_if_fail (bmp != NULL, COGL_INVALID_HANDLE);
 
   internal_format =
@@ -331,8 +341,8 @@ _cogl_texture_2d_new_from_bitmap (CoglBitmap      *bmp,
       _cogl_bitmap_unmap (dst_bmp);
     }
 
-  _cogl_texture_driver_gen (GL_TEXTURE_2D, 1, &tex_2d->gl_texture);
-  _cogl_texture_driver_upload_to_gl (GL_TEXTURE_2D,
+  ctx->texture_driver->gen (GL_TEXTURE_2D, 1, &tex_2d->gl_texture);
+  ctx->texture_driver->upload_to_gl (GL_TEXTURE_2D,
                                      tex_2d->gl_texture,
                                      FALSE,
                                      dst_bmp,
@@ -401,55 +411,57 @@ cogl_texture_2d_new_from_foreign (CoglContext *ctx,
   GLenum         gl_int_format = 0;
   CoglTexture2D *tex_2d;
 
-  if (!_cogl_texture_driver_allows_foreign_gl_target (GL_TEXTURE_2D))
+  if (!ctx->texture_driver->allows_foreign_gl_target (GL_TEXTURE_2D))
     return COGL_INVALID_HANDLE;
 
   /* Make sure it is a valid GL texture object */
-  if (!glIsTexture (gl_handle))
+  if (!ctx->glIsTexture (gl_handle))
     return COGL_INVALID_HANDLE;
 
   /* Make sure binding succeeds */
-  while ((gl_error = glGetError ()) != GL_NO_ERROR)
+  while ((gl_error = ctx->glGetError ()) != GL_NO_ERROR)
     ;
 
   _cogl_bind_gl_texture_transient (GL_TEXTURE_2D, gl_handle, TRUE);
-  if (glGetError () != GL_NO_ERROR)
+  if (ctx->glGetError () != GL_NO_ERROR)
     return COGL_INVALID_HANDLE;
 
   /* Obtain texture parameters
      (only level 0 we are interested in) */
 
 #if HAVE_COGL_GL
+  if (ctx->driver == COGL_DRIVER_GL)
+    {
+      GE( ctx, glGetTexLevelParameteriv (GL_TEXTURE_2D, 0,
+                                         GL_TEXTURE_COMPRESSED,
+                                         &gl_compressed) );
 
-  GE( glGetTexLevelParameteriv (GL_TEXTURE_2D, 0,
-                                GL_TEXTURE_COMPRESSED,
-                                &gl_compressed) );
+      {
+        GLint val;
 
-  {
-    GLint val;
+        GE( ctx, glGetTexLevelParameteriv (GL_TEXTURE_2D, 0,
+                                           GL_TEXTURE_INTERNAL_FORMAT,
+                                           &val) );
 
-    GE( glGetTexLevelParameteriv (GL_TEXTURE_2D, 0,
-                                  GL_TEXTURE_INTERNAL_FORMAT,
-                                  &val) );
-
-    gl_int_format = val;
-  }
-
-  /* If we can query GL for the actual pixel format then we'll ignore
-     the passed in format and use that. */
-  if (!_cogl_pixel_format_from_gl_internal (gl_int_format, &format))
-    return COGL_INVALID_HANDLE;
-
-#else
-
-  /* Otherwise we'll assume we can derive the GL format from the
-     passed in format */
-  _cogl_pixel_format_to_gl (format,
-                            &gl_int_format,
-                            NULL,
-                            NULL);
+        gl_int_format = val;
+      }
 
+      /* If we can query GL for the actual pixel format then we'll ignore
+         the passed in format and use that. */
+      if (!ctx->texture_driver->pixel_format_from_gl_internal (gl_int_format,
+                                                               &format))
+        return COGL_INVALID_HANDLE;
+    }
+  else
 #endif
+    {
+      /* Otherwise we'll assume we can derive the GL format from the
+         passed in format */
+      ctx->texture_driver->pixel_format_to_gl (format,
+                                               &gl_int_format,
+                                               NULL,
+                                               NULL);
+    }
 
   /* Note: We always trust the given width and height without querying
    * the texture object because the user may be creating a Cogl
@@ -524,15 +536,15 @@ _cogl_egl_texture_2d_new_from_image (CoglContext *ctx,
   tex_2d = _cogl_texture_2d_create_base (width, height, COGL_TEXTURE_NONE,
                                          format);
 
-  _cogl_texture_driver_gen (GL_TEXTURE_2D, 1, &tex_2d->gl_texture);
+  ctx->texture_driver->gen (GL_TEXTURE_2D, 1, &tex_2d->gl_texture);
   _cogl_bind_gl_texture_transient (GL_TEXTURE_2D,
                                    tex_2d->gl_texture,
                                    FALSE);
 
-  while ((gl_error = glGetError ()) != GL_NO_ERROR)
+  while ((gl_error = ctx->glGetError ()) != GL_NO_ERROR)
     ;
-  ctx->drv.pf_glEGLImageTargetTexture2D (GL_TEXTURE_2D, image);
-  if (glGetError () != GL_NO_ERROR)
+  ctx->glEGLImageTargetTexture2D (GL_TEXTURE_2D, image);
+  if (ctx->glGetError () != GL_NO_ERROR)
     {
       g_set_error (error,
                    COGL_TEXTURE_ERROR,
@@ -630,6 +642,8 @@ _cogl_texture_2d_copy_from_framebuffer (CoglHandle handle,
 {
   CoglTexture2D *tex_2d;
 
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
   g_return_if_fail (cogl_is_texture_2d (handle));
 
   tex_2d = COGL_TEXTURE_2D (handle);
@@ -644,11 +658,11 @@ _cogl_texture_2d_copy_from_framebuffer (CoglHandle handle,
                                    tex_2d->gl_texture,
                                    tex_2d->is_foreign);
 
-  glCopyTexSubImage2D (GL_TEXTURE_2D,
-                       0, /* level */
-                       dst_x, dst_y,
-                       src_x, src_y,
-                       width, height);
+  ctx->glCopyTexSubImage2D (GL_TEXTURE_2D,
+                            0, /* level */
+                            dst_x, dst_y,
+                            src_x, src_y,
+                            width, height);
 
   tex_2d->mipmaps_dirty = TRUE;
 }
@@ -721,6 +735,8 @@ _cogl_texture_2d_set_filters (CoglTexture *tex,
 {
   CoglTexture2D *tex_2d = COGL_TEXTURE_2D (tex);
 
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
   if (min_filter == tex_2d->min_filter
       && mag_filter == tex_2d->mag_filter)
     return;
@@ -733,8 +749,8 @@ _cogl_texture_2d_set_filters (CoglTexture *tex,
   _cogl_bind_gl_texture_transient (GL_TEXTURE_2D,
                                    tex_2d->gl_texture,
                                    tex_2d->is_foreign);
-  GE( glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, mag_filter) );
-  GE( glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, min_filter) );
+  GE( ctx, glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, mag_filter) );
+  GE( ctx, glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, min_filter) );
 }
 
 static void
@@ -756,20 +772,20 @@ _cogl_texture_2d_pre_paint (CoglTexture *tex, CoglTexturePrePaintFlags flags)
          available we'll fallback to temporarily enabling
          GL_GENERATE_MIPMAP and reuploading the first pixel */
       if (cogl_features_available (COGL_FEATURE_OFFSCREEN))
-        _cogl_texture_driver_gl_generate_mipmaps (GL_TEXTURE_2D);
-#ifndef HAVE_COGL_GLES2
+        ctx->texture_driver->gl_generate_mipmaps (GL_TEXTURE_2D);
+#if defined(HAVE_COGL_GLES) || defined(HAVE_COGL_GL)
       else
         {
-          GE( glTexParameteri (GL_TEXTURE_2D,
-                               GL_GENERATE_MIPMAP,
-                               GL_TRUE) );
-          GE( glTexSubImage2D (GL_TEXTURE_2D, 0, 0, 0, 1, 1,
-                               tex_2d->first_pixel.gl_format,
-                               tex_2d->first_pixel.gl_type,
-                               tex_2d->first_pixel.data) );
-          GE( glTexParameteri (GL_TEXTURE_2D,
-                               GL_GENERATE_MIPMAP,
-                               GL_FALSE) );
+          GE( ctx, glTexParameteri (GL_TEXTURE_2D,
+                                    GL_GENERATE_MIPMAP,
+                                    GL_TRUE) );
+          GE( ctx, glTexSubImage2D (GL_TEXTURE_2D, 0, 0, 0, 1, 1,
+                                    tex_2d->first_pixel.gl_format,
+                                    tex_2d->first_pixel.gl_type,
+                                    tex_2d->first_pixel.data) );
+          GE( ctx, glTexParameteri (GL_TEXTURE_2D,
+                                    GL_GENERATE_MIPMAP,
+                                    GL_FALSE) );
         }
 #endif
 
@@ -798,10 +814,14 @@ _cogl_texture_2d_set_region (CoglTexture    *tex,
   GLenum         gl_type;
   guint8        *data;
 
-  _cogl_pixel_format_to_gl (_cogl_bitmap_get_format (bmp),
-                            NULL, /* internal format */
-                            &gl_format,
-                            &gl_type);
+  _COGL_GET_CONTEXT (ctx, FALSE);
+
+  bmp = _cogl_texture_prepare_for_upload (bmp,
+                                          cogl_texture_get_format (tex),
+                                          NULL,
+                                          NULL,
+                                          &gl_format,
+                                          &gl_type);
 
   /* If this touches the first pixel then we'll update our copy */
   if (dst_x == 0 && dst_y == 0 &&
@@ -820,7 +840,7 @@ _cogl_texture_2d_set_region (CoglTexture    *tex,
     }
 
   /* Send data to GL */
-  _cogl_texture_driver_upload_subregion_to_gl (GL_TEXTURE_2D,
+  ctx->texture_driver->upload_subregion_to_gl (GL_TEXTURE_2D,
                                                tex_2d->gl_texture,
                                                FALSE,
                                                src_x, src_y,
@@ -832,6 +852,8 @@ _cogl_texture_2d_set_region (CoglTexture    *tex,
 
   tex_2d->mipmaps_dirty = TRUE;
 
+  cogl_object_unref (bmp);
+
   return TRUE;
 }
 
@@ -846,19 +868,21 @@ _cogl_texture_2d_get_data (CoglTexture     *tex,
   GLenum           gl_format;
   GLenum           gl_type;
 
+  _COGL_GET_CONTEXT (ctx, FALSE);
+
   bpp = _cogl_get_format_bpp (format);
 
-  _cogl_pixel_format_to_gl (format,
-                            NULL, /* internal format */
-                            &gl_format,
-                            &gl_type);
+  ctx->texture_driver->pixel_format_to_gl (format,
+                                           NULL, /* internal format */
+                                           &gl_format,
+                                           &gl_type);
 
-  _cogl_texture_driver_prep_gl_for_pixels_download (rowstride, bpp);
+  ctx->texture_driver->prep_gl_for_pixels_download (rowstride, bpp);
 
   _cogl_bind_gl_texture_transient (GL_TEXTURE_2D,
                                    tex_2d->gl_texture,
                                    tex_2d->is_foreign);
-  return _cogl_texture_driver_gl_get_tex_image (GL_TEXTURE_2D,
+  return ctx->texture_driver->gl_get_tex_image (GL_TEXTURE_2D,
                                                 gl_format,
                                                 gl_type,
                                                 data);
