@@ -39,8 +39,8 @@
 #include "cogl-renderer-private.h"
 #include "cogl-onscreen-template-private.h"
 #ifdef COGL_HAS_EGL_PLATFORM_POWERVR_X11_SUPPORT
-#include "cogl-renderer-xlib-private.h"
-#include "cogl-display-xlib-private.h"
+#include "cogl-xlib-renderer-private.h"
+#include "cogl-xlib-display-private.h"
 #endif
 
 #ifdef COGL_HAS_XLIB_SUPPORT
@@ -82,7 +82,7 @@ typedef enum _CoglEGLWinsysFeature
 typedef struct _CoglRendererEGL
 {
 #ifdef COGL_HAS_EGL_PLATFORM_POWERVR_X11_SUPPORT
-  CoglRendererXlib _parent;
+  CoglXlibRenderer _parent;
 #endif
 
   CoglEGLWinsysFeature private_features;
@@ -120,7 +120,7 @@ typedef struct _CoglRendererEGL
 typedef struct _CoglDisplayEGL
 {
 #ifdef COGL_HAS_EGL_PLATFORM_POWERVR_X11_SUPPORT
-  CoglDisplayXlib _parent;
+  CoglXlibDisplay _parent;
 #endif
 
   EGLContext egl_context;
@@ -197,7 +197,7 @@ typedef struct _CoglTexturePixmapEGL
 #undef COGL_WINSYS_FEATURE_BEGIN
 #define COGL_WINSYS_FEATURE_BEGIN(name, namespaces, extension_names,    \
                                   egl_private_flags)                    \
-  { 255, 255, namespaces, extension_names,                              \
+  { 255, 255, 0, namespaces, extension_names,                           \
       0, egl_private_flags,                                             \
       0,                                                                \
       cogl_egl_feature_ ## name ## _funcs },
@@ -212,9 +212,19 @@ static const CoglFeatureData winsys_feature_data[] =
   };
 
 static CoglFuncPtr
-_cogl_winsys_get_proc_address (const char *name)
+_cogl_winsys_renderer_get_proc_address (CoglRenderer *renderer,
+                                        const char *name)
 {
-  return (CoglFuncPtr) eglGetProcAddress (name);
+  void *ptr;
+
+  ptr = eglGetProcAddress (name);
+
+  /* eglGetProcAddress doesn't support fetching core API so we need to
+     get that separately with GModule */
+  if (ptr == NULL)
+    g_module_symbol (renderer->libgl_module, name, &ptr);
+
+  return ptr;
 }
 
 #ifdef COGL_HAS_EGL_PLATFORM_ANDROID_SUPPORT
@@ -223,6 +233,8 @@ static ANativeWindow *android_native_window;
 void
 cogl_android_set_native_window (ANativeWindow *window)
 {
+  _cogl_init ();
+
   android_native_window = window;
 }
 #endif
@@ -250,9 +262,8 @@ find_onscreen_for_xid (CoglContext *context, guint32 xid)
 }
 
 static CoglFilterReturn
-event_filter_cb (void *event, void *data)
+event_filter_cb (XEvent *xevent, void *data)
 {
-  XEvent *xevent = event;
   CoglContext *context = data;
 
   if (xevent->type == ConfigureNotify)
@@ -285,7 +296,7 @@ _cogl_winsys_renderer_disconnect (CoglRenderer *renderer)
 #endif
 
 #ifdef COGL_HAS_EGL_PLATFORM_POWERVR_X11_SUPPORT
-  _cogl_renderer_xlib_disconnect (renderer);
+  _cogl_xlib_renderer_disconnect (renderer);
 #endif
 
   eglTerminate (egl_renderer->edpy);
@@ -341,7 +352,6 @@ static void
 check_egl_extensions (CoglRenderer *renderer)
 {
   CoglRendererEGL *egl_renderer = renderer->winsys;
-  const CoglWinsysVtable *winsys = renderer->winsys_vtable;
   const char *egl_extensions;
   int i;
 
@@ -351,13 +361,14 @@ check_egl_extensions (CoglRenderer *renderer)
 
   egl_renderer->private_features = 0;
   for (i = 0; i < G_N_ELEMENTS (winsys_feature_data); i++)
-    if (_cogl_feature_check (winsys,
+    if (_cogl_feature_check (renderer,
                              "EGL", winsys_feature_data + i, 0, 0,
+                             COGL_DRIVER_GL, /* the driver isn't used */
                              egl_extensions,
                              egl_renderer))
       {
         egl_renderer->private_features |=
-          winsys_feature_data[0].feature_flags_private;
+          winsys_feature_data[i].feature_flags_private;
       }
 }
 
@@ -367,7 +378,7 @@ _cogl_winsys_renderer_connect (CoglRenderer *renderer,
 {
   CoglRendererEGL *egl_renderer;
 #ifdef COGL_HAS_EGL_PLATFORM_POWERVR_X11_SUPPORT
-  CoglRendererXlib *xlib_renderer;
+  CoglXlibRenderer *xlib_renderer;
 #endif
   EGLBoolean status;
 #ifdef COGL_HAS_EGL_PLATFORM_GDL_SUPPORT
@@ -381,7 +392,7 @@ _cogl_winsys_renderer_connect (CoglRenderer *renderer,
 #ifdef COGL_HAS_EGL_PLATFORM_POWERVR_X11_SUPPORT
   xlib_renderer = renderer->winsys;
 
-  if (!_cogl_renderer_xlib_connect (renderer, error))
+  if (!_cogl_xlib_renderer_connect (renderer, error))
     goto error;
 
   egl_renderer->edpy =
@@ -501,19 +512,20 @@ error:
   return FALSE;
 }
 
-static void
-update_winsys_features (CoglContext *context)
+static gboolean
+update_winsys_features (CoglContext *context, GError **error)
 {
   CoglDisplayEGL *egl_display = context->display->winsys;
   CoglRendererEGL *egl_renderer = context->display->renderer->winsys;
 
-  g_return_if_fail (egl_display->egl_context);
+  g_return_val_if_fail (egl_display->egl_context, FALSE);
 
   memset (context->winsys_features, 0, sizeof (context->winsys_features));
 
   check_egl_extensions (context->display->renderer);
 
-  _cogl_gl_update_features (context);
+  if (!_cogl_context_update_features (context, error))
+    return FALSE;
 
 #if defined (COGL_HAS_EGL_PLATFORM_POWERVR_X11_SUPPORT) || \
     defined (COGL_HAS_EGL_PLATFORM_WAYLAND_SUPPORT)
@@ -530,13 +542,15 @@ update_winsys_features (CoglContext *context)
       COGL_FLAGS_SET (context->winsys_features,
                       COGL_WINSYS_FEATURE_SWAP_REGION_THROTTLE, TRUE);
     }
+
+  return TRUE;
 }
 
 #ifdef COGL_HAS_EGL_PLATFORM_POWERVR_X11_SUPPORT
 static XVisualInfo *
 get_visual_info (CoglDisplay *display, EGLConfig egl_config)
 {
-  CoglRendererXlib *xlib_renderer = display->renderer->winsys;
+  CoglXlibRenderer *xlib_renderer = display->renderer->winsys;
   CoglRendererEGL *egl_renderer = display->renderer->winsys;
   XVisualInfo visinfo_template;
   int template_mask = 0;
@@ -590,8 +604,8 @@ try_create_context (CoglDisplay *display,
 {
   CoglDisplayEGL *egl_display = display->winsys;
 #ifdef COGL_HAS_EGL_PLATFORM_POWERVR_X11_SUPPORT
-  CoglDisplayXlib *xlib_display = display->winsys;
-  CoglRendererXlib *xlib_renderer = display->renderer->winsys;
+  CoglXlibDisplay *xlib_display = display->winsys;
+  CoglXlibRenderer *xlib_renderer = display->renderer->winsys;
 #endif
   CoglRendererEGL *egl_renderer = display->renderer->winsys;
   EGLDisplay edpy;
@@ -618,23 +632,17 @@ try_create_context (CoglDisplay *display,
 
     EGL_BUFFER_SIZE,     EGL_DONT_CARE,
 
-#if defined (HAVE_COGL_GL)
-    EGL_RENDERABLE_TYPE, EGL_OPENGL_BIT,
-#elif defined (HAVE_COGL_GLES2)
-    EGL_RENDERABLE_TYPE, EGL_OPENGL_ES2_BIT,
-#else
-    EGL_RENDERABLE_TYPE, EGL_OPENGL_ES_BIT,
-#endif
+    EGL_RENDERABLE_TYPE, (display->renderer->driver == COGL_DRIVER_GL ?
+                          EGL_OPENGL_BIT :
+                          display->renderer->driver == COGL_DRIVER_GLES1 ?
+                          EGL_OPENGL_ES_BIT :
+                          EGL_OPENGL_ES2_BIT),
 
     EGL_SURFACE_TYPE,    EGL_WINDOW_BIT,
 
     EGL_NONE
   };
-#if defined (HAVE_COGL_GLES2)
-  EGLint attribs[] = { EGL_CONTEXT_CLIENT_VERSION, 2, EGL_NONE };
-#else
-  EGLint *attribs = NULL;
-#endif
+  EGLint attribs[3];
 
 #ifdef COGL_HAS_EGL_PLATFORM_POWERVR_X11_SUPPORT
   XVisualInfo *xvisinfo;
@@ -645,11 +653,19 @@ try_create_context (CoglDisplay *display,
 #endif
   const char *error_message;
 
+  if (display->renderer->driver == COGL_DRIVER_GLES2)
+    {
+      attribs[0] = EGL_CONTEXT_CLIENT_VERSION;
+      attribs[1] = 2;
+      attribs[2] = EGL_NONE;
+    }
+  else
+    attribs[0] = EGL_NONE;
+
   edpy = egl_renderer->edpy;
 
-#ifdef HAVE_COGL_GL
-  eglBindAPI (EGL_OPENGL_API);
-#endif
+  if (display->renderer->driver == COGL_DRIVER_GL)
+    eglBindAPI (EGL_OPENGL_API);
 
   /* Some GLES hardware can't support a stencil buffer: */
   if (retry_cookie == 1)
@@ -829,6 +845,40 @@ try_create_context (CoglDisplay *display,
                      EGL_HEIGHT,
                      &egl_display->egl_surface_height);
   }
+
+#elif defined (COGL_HAS_EGL_PLATFORM_GDL_SUPPORT)
+
+  egl_display->egl_surface =
+    eglCreateWindowSurface (edpy,
+                            config,
+                            (NativeWindowType) display->gdl_plane,
+                            NULL);
+
+  if (egl_display->egl_surface == EGL_NO_SURFACE)
+    {
+      error_message = "Unable to create EGL window surface";
+      goto fail;
+    }
+
+  if (!eglMakeCurrent (egl_renderer->edpy,
+                       egl_display->egl_surface,
+                       egl_display->egl_surface,
+                       egl_display->egl_context))
+    {
+      error_message = "Unable to eglMakeCurrent with egl surface";
+      goto fail;
+    }
+
+  eglQuerySurface (egl_renderer->edpy,
+                   egl_display->egl_surface,
+                   EGL_WIDTH,
+                   &egl_display->egl_surface_width);
+
+  eglQuerySurface (egl_renderer->edpy,
+                   egl_display->egl_surface,
+                   EGL_HEIGHT,
+                   &egl_display->egl_surface_height);
+
 #elif defined (COGL_HAS_EGL_PLATFORM_POWERVR_NULL_SUPPORT)
 
   egl_display->egl_surface =
@@ -891,8 +941,8 @@ cleanup_context (CoglDisplay *display)
   CoglDisplayEGL *egl_display = display->winsys;
   CoglRendererEGL *egl_renderer = display->renderer->winsys;
 #ifdef COGL_HAS_EGL_PLATFORM_POWERVR_X11_SUPPORT
-  CoglDisplayXlib *xlib_display = display->winsys;
-  CoglRendererXlib *xlib_renderer = display->renderer->winsys;
+  CoglXlibDisplay *xlib_display = display->winsys;
+  CoglXlibRenderer *xlib_renderer = display->renderer->winsys;
 #endif
 
   if (egl_display->egl_context != EGL_NO_CONTEXT)
@@ -903,7 +953,14 @@ cleanup_context (CoglDisplay *display)
       egl_display->egl_context = EGL_NO_CONTEXT;
     }
 
-#ifdef COGL_HAS_EGL_PLATFORM_POWERVR_X11_SUPPORT
+#if defined (COGL_HAS_EGL_PLATFORM_POWERVR_NULL_SUPPORT) || \
+    defined (COGL_HAS_EGL_PLATFORM_GDL_SUPPORT)
+  if (egl_display->egl_surface != EGL_NO_SURFACE)
+    {
+      eglDestroySurface (egl_renderer->edpy, egl_display->egl_surface);
+      egl_display->egl_surface = EGL_NO_SURFACE;
+    }
+#elif COGL_HAS_EGL_PLATFORM_POWERVR_X11_SUPPORT
   if (egl_display->dummy_surface != EGL_NO_SURFACE)
     {
       eglDestroySurface (egl_renderer->edpy, egl_display->dummy_surface);
@@ -948,8 +1005,7 @@ create_context (CoglDisplay *display, GError **error)
 
   g_return_val_if_fail (egl_display->egl_context == NULL, TRUE);
 
-  if (display->onscreen_template &&
-      display->onscreen_template->swap_chain &&
+  if (display->onscreen_template->swap_chain &&
       display->onscreen_template->swap_chain->has_alpha)
     support_transparent_windows = TRUE;
   else
@@ -991,6 +1047,7 @@ gdl_plane_init (CoglDisplay *display, GError **error)
 {
   gboolean ret = TRUE;
   gdl_color_space_t colorSpace = GDL_COLOR_SPACE_RGB;
+  gdl_pixel_format_t pixfmt = GDL_PF_ARGB_32;
   gdl_rectangle_t dstRect;
   gdl_display_info_t display_info;
   gdl_ret_t rc = GDL_SUCCESS;
@@ -1027,9 +1084,9 @@ gdl_plane_init (CoglDisplay *display, GError **error)
   dstRect.height = display_info.tvmode.height;
 
   /* Configure the plane attribute. */
-  rc = gdl_plane_reset (plane);
+  rc = gdl_plane_reset (display->gdl_plane);
   if (rc == GDL_SUCCESS)
-    rc = gdl_plane_config_begin (plane);
+    rc = gdl_plane_config_begin (display->gdl_plane);
 
   if (rc == GDL_SUCCESS)
     rc = gdl_plane_set_attr (GDL_PLANE_SRC_COLOR_SPACE, &colorSpace);
@@ -1040,9 +1097,17 @@ gdl_plane_init (CoglDisplay *display, GError **error)
   if (rc == GDL_SUCCESS)
     rc = gdl_plane_set_attr (GDL_PLANE_DST_RECT, &dstRect);
 
+  /* Default to triple buffering if the swap_chain doesn't have an explicit
+   * length */
   if (rc == GDL_SUCCESS)
-    rc = gdl_plane_set_uint (GDL_PLANE_NUM_GFX_SURFACES,
-                             display->swap_chain->length);
+    {
+      if (display->onscreen_template->swap_chain &&
+          display->onscreen_template->swap_chain->length != -1)
+        rc = gdl_plane_set_uint (GDL_PLANE_NUM_GFX_SURFACES,
+                                 display->onscreen_template->swap_chain->length);
+      else
+        rc = gdl_plane_set_uint (GDL_PLANE_NUM_GFX_SURFACES, 3);
+    }
 
   if (rc == GDL_SUCCESS)
     rc = gdl_plane_config_end (GDL_FALSE);
@@ -1057,6 +1122,8 @@ gdl_plane_init (CoglDisplay *display, GError **error)
     }
 
   gdl_close ();
+
+  return TRUE;
 }
 #endif
 
@@ -1106,22 +1173,20 @@ _cogl_winsys_context_init (CoglContext *context, GError **error)
   context->winsys = g_new0 (CoglContextEGL, 1);
 
 #ifdef COGL_HAS_EGL_PLATFORM_POWERVR_X11_SUPPORT
-  cogl_renderer_add_native_filter (context->display->renderer,
-                                   event_filter_cb,
-                                   context);
+  cogl_xlib_renderer_add_filter (context->display->renderer,
+                                 event_filter_cb,
+                                 context);
 #endif
-  update_winsys_features (context);
-
-  return TRUE;
+  return update_winsys_features (context, error);
 }
 
 static void
 _cogl_winsys_context_deinit (CoglContext *context)
 {
 #ifdef COGL_HAS_EGL_PLATFORM_POWERVR_X11_SUPPORT
-  cogl_renderer_remove_native_filter (context->display->renderer,
-                                      event_filter_cb,
-                                      context);
+  cogl_xlib_renderer_remove_filter (context->display->renderer,
+                                    event_filter_cb,
+                                    context);
 #endif
   g_free (context->winsys);
 }
@@ -1136,7 +1201,7 @@ _cogl_winsys_onscreen_init (CoglOnscreen *onscreen,
   CoglDisplayEGL *egl_display = display->winsys;
 #ifdef COGL_HAS_EGL_PLATFORM_POWERVR_X11_SUPPORT
   CoglRendererEGL *egl_renderer = display->renderer->winsys;
-  CoglRendererXlib *xlib_renderer = display->renderer->winsys;
+  CoglXlibRenderer *xlib_renderer = display->renderer->winsys;
   CoglOnscreenXlib *xlib_onscreen;
   Window xwin;
 #endif
@@ -1170,10 +1235,10 @@ _cogl_winsys_onscreen_init (CoglOnscreen *onscreen,
 
       xwin = onscreen->foreign_xid;
 
-      _cogl_renderer_xlib_trap_errors (display->renderer, &state);
+      _cogl_xlib_renderer_trap_errors (display->renderer, &state);
 
       status = XGetWindowAttributes (xlib_renderer->xdpy, xwin, &attr);
-      xerror = _cogl_renderer_xlib_untrap_errors (display->renderer, &state);
+      xerror = _cogl_xlib_renderer_untrap_errors (display->renderer, &state);
       if (status == 0 || xerror)
         {
           char message[1000];
@@ -1207,7 +1272,7 @@ _cogl_winsys_onscreen_init (CoglOnscreen *onscreen,
       width = cogl_framebuffer_get_width (framebuffer);
       height = cogl_framebuffer_get_height (framebuffer);
 
-      _cogl_renderer_xlib_trap_errors (display->renderer, &state);
+      _cogl_xlib_renderer_trap_errors (display->renderer, &state);
 
       xvisinfo = get_visual_info (display, egl_display->egl_config);
       if (xvisinfo == NULL)
@@ -1245,7 +1310,7 @@ _cogl_winsys_onscreen_init (CoglOnscreen *onscreen,
       XFree (xvisinfo);
 
       XSync (xlib_renderer->xdpy, False);
-      xerror = _cogl_renderer_xlib_untrap_errors (display->renderer, &state);
+      xerror = _cogl_xlib_renderer_untrap_errors (display->renderer, &state);
       if (xerror)
         {
           char message[1000];
@@ -1312,7 +1377,8 @@ _cogl_winsys_onscreen_init (CoglOnscreen *onscreen,
   wl_surface_map_toplevel (egl_onscreen->wayland_surface);
 
 #elif defined (COGL_HAS_EGL_PLATFORM_POWERVR_NULL_SUPPORT) || \
-      defined (COGL_HAS_EGL_PLATFORM_ANDROID_SUPPORT)
+      defined (COGL_HAS_EGL_PLATFORM_ANDROID_SUPPORT)      || \
+      defined (COGL_HAS_EGL_PLATFORM_GDL_SUPPORT)
   if (egl_display->have_onscreen)
     {
       g_set_error (error, COGL_WINSYS_ERROR,
@@ -1340,7 +1406,7 @@ _cogl_winsys_onscreen_deinit (CoglOnscreen *onscreen)
   CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (onscreen);
   CoglContext *context = framebuffer->context;
 #ifdef COGL_HAS_EGL_PLATFORM_POWERVR_X11_SUPPORT
-  CoglRendererXlib *xlib_renderer = context->display->renderer->winsys;
+  CoglXlibRenderer *xlib_renderer = context->display->renderer->winsys;
   CoglXlibTrapState old_state;
   CoglOnscreenXlib *xlib_onscreen = onscreen->winsys;
 #elif defined (COGL_HAS_EGL_PLATFORM_POWERVR_NULL_SUPPORT)
@@ -1366,7 +1432,7 @@ _cogl_winsys_onscreen_deinit (CoglOnscreen *onscreen)
 #endif
 
 #ifdef COGL_HAS_EGL_PLATFORM_POWERVR_X11_SUPPORT
-  _cogl_renderer_xlib_trap_errors (context->display->renderer, &old_state);
+  _cogl_xlib_renderer_trap_errors (context->display->renderer, &old_state);
 
   if (!xlib_onscreen->is_foreign_xwin && xlib_onscreen->xwin != None)
     {
@@ -1378,7 +1444,7 @@ _cogl_winsys_onscreen_deinit (CoglOnscreen *onscreen)
 
   XSync (xlib_renderer->xdpy, False);
 
-  if (_cogl_renderer_xlib_untrap_errors (context->display->renderer,
+  if (_cogl_xlib_renderer_untrap_errors (context->display->renderer,
                                          &old_state) != Success)
     g_warning ("X Error while destroying X window");
 #endif
@@ -1423,7 +1489,8 @@ _cogl_winsys_onscreen_bind (CoglOnscreen *onscreen)
                       egl_display->egl_context);
       egl_context->current_surface = egl_display->dummy_surface;
 #elif defined (COGL_HAS_EGL_PLATFORM_POWERVR_NULL_SUPPORT) || \
-      defined (COGL_HAS_EGL_PLATFORM_ANDROID_SUPPORT)
+      defined (COGL_HAS_EGL_PLATFORM_ANDROID_SUPPORT)      || \
+      defined (COGL_HAS_EGL_PLATFORM_GDL_SUPPORT)
       return;
 #else
 #error "Unknown EGL platform"
@@ -1446,12 +1513,26 @@ _cogl_winsys_onscreen_bind (CoglOnscreen *onscreen)
 
 static void
 _cogl_winsys_onscreen_swap_region (CoglOnscreen *onscreen,
-                                   int *rectangles,
+                                   const int *user_rectangles,
                                    int n_rectangles)
 {
   CoglContext *context = COGL_FRAMEBUFFER (onscreen)->context;
   CoglRendererEGL *egl_renderer = context->display->renderer->winsys;
   CoglOnscreenEGL *egl_onscreen = onscreen->winsys;
+  CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (onscreen);
+  int framebuffer_height  = cogl_framebuffer_get_height (framebuffer);
+  int *rectangles = g_alloca (sizeof (int) * n_rectangles * 4);
+  int i;
+
+  /* eglSwapBuffersRegion expects rectangles relative to the bottom left corner
+   * but we are given rectangles relative to the top left so we need to flip
+   * them... */
+  memcpy (rectangles, user_rectangles, sizeof (int) * n_rectangles * 4);
+  for (i = 0; i < n_rectangles; i++)
+    {
+      int *rect = &rectangles[4 * i];
+      rect[1] = framebuffer_height - rect[1] - rect[3];
+    }
 
   if (egl_renderer->pf_eglSwapBuffersRegion (egl_renderer->edpy,
                                              egl_onscreen->egl_surface,
@@ -1466,7 +1547,7 @@ _cogl_winsys_onscreen_set_visibility (CoglOnscreen *onscreen,
                                       gboolean visibility)
 {
   CoglContext *context = COGL_FRAMEBUFFER (onscreen)->context;
-  CoglRendererXlib *xlib_renderer = context->display->renderer->winsys;
+  CoglXlibRenderer *xlib_renderer = context->display->renderer->winsys;
   CoglOnscreenXlib *xlib_onscreen = onscreen->winsys;
 
   if (visibility)
@@ -1552,13 +1633,16 @@ _cogl_winsys_texture_pixmap_x11_create (CoglTexturePixmapX11 *tex_pixmap)
   CoglTexturePixmapEGL *egl_tex_pixmap;
   EGLint attribs[] = {EGL_IMAGE_PRESERVED_KHR, EGL_TRUE, EGL_NONE};
   CoglPixelFormat texture_format;
+  CoglRendererEGL *egl_renderer;
 
   /* FIXME: It should be possible to get to a CoglContext from any
    * CoglTexture pointer. */
   _COGL_GET_CONTEXT (ctx, FALSE);
 
-  if (!(ctx->private_feature_flags &
-        COGL_PRIVATE_FEATURE_EGL_IMAGE_FROM_X11_PIXMAP) ||
+  egl_renderer = ctx->display->renderer->winsys;
+
+  if (!(egl_renderer->private_features &
+        COGL_EGL_WINSYS_FEATURE_EGL_IMAGE_FROM_X11_PIXMAP) ||
       !(ctx->private_feature_flags &
         COGL_PRIVATE_FEATURE_TEXTURE_2D_FROM_EGL_IMAGE))
     {
@@ -1574,7 +1658,10 @@ _cogl_winsys_texture_pixmap_x11_create (CoglTexturePixmapX11 *tex_pixmap)
                             (EGLClientBuffer)tex_pixmap->pixmap,
                             attribs);
   if (egl_tex_pixmap->image == EGL_NO_IMAGE_KHR)
-    return FALSE;
+    {
+      g_free (egl_tex_pixmap);
+      return FALSE;
+    }
 
   texture_format = (tex_pixmap->depth >= 32 ?
                     COGL_PIXEL_FORMAT_RGBA_8888_PRE :
@@ -1644,8 +1731,9 @@ _cogl_winsys_texture_pixmap_x11_get_texture (CoglTexturePixmapX11 *tex_pixmap)
 
 static CoglWinsysVtable _cogl_winsys_vtable =
   {
+    .id = COGL_WINSYS_ID_EGL,
     .name = "EGL",
-    .get_proc_address = _cogl_winsys_get_proc_address,
+    .renderer_get_proc_address = _cogl_winsys_renderer_get_proc_address,
     .renderer_connect = _cogl_winsys_renderer_connect,
     .renderer_disconnect = _cogl_winsys_renderer_disconnect,
     .display_setup = _cogl_winsys_display_setup,
@@ -1707,7 +1795,7 @@ _cogl_winsys_egl_get_vtable (void)
  * points... */
 #ifdef COGL_HAS_EGL_PLATFORM_WAYLAND_SUPPORT
 void
-cogl_renderer_wayland_set_foreign_display (CoglRenderer *renderer,
+cogl_wayland_renderer_set_foreign_display (CoglRenderer *renderer,
                                            struct wl_display *display)
 {
   g_return_if_fail (cogl_is_renderer (renderer));
@@ -1719,7 +1807,7 @@ cogl_renderer_wayland_set_foreign_display (CoglRenderer *renderer,
 }
 
 struct wl_display *
-cogl_renderer_wayland_get_display (CoglRenderer *renderer)
+cogl_wayland_renderer_get_display (CoglRenderer *renderer)
 {
   g_return_val_if_fail (cogl_is_renderer (renderer), NULL);
 
@@ -1735,7 +1823,7 @@ cogl_renderer_wayland_get_display (CoglRenderer *renderer)
 }
 
 void
-cogl_renderer_wayland_set_foreign_compositor (CoglRenderer *renderer,
+cogl_wayland_renderer_set_foreign_compositor (CoglRenderer *renderer,
                                               struct wl_compositor *compositor)
 {
   g_return_if_fail (cogl_is_renderer (renderer));
@@ -1747,7 +1835,7 @@ cogl_renderer_wayland_set_foreign_compositor (CoglRenderer *renderer,
 }
 
 struct wl_compositor *
-cogl_renderer_wayland_get_compositor (CoglRenderer *renderer)
+cogl_wayland_renderer_get_compositor (CoglRenderer *renderer)
 {
   g_return_val_if_fail (cogl_is_renderer (renderer), NULL);
 
@@ -1788,11 +1876,21 @@ _cogl_egl_create_image (CoglContext *ctx,
 {
   CoglDisplayEGL *egl_display = ctx->display->winsys;
   CoglRendererEGL *egl_renderer = ctx->display->renderer->winsys;
+  EGLContext egl_ctx;
 
   g_return_val_if_fail (egl_renderer->pf_eglCreateImage, EGL_NO_IMAGE_KHR);
 
+  /* The EGL_KHR_image_pixmap spec explicitly states that EGL_NO_CONTEXT must
+   * always be used in conjunction with the EGL_NATIVE_PIXMAP_KHR target */
+#ifdef EGL_KHR_image_pixmap
+  if (target == EGL_NATIVE_PIXMAP_KHR)
+    egl_ctx = EGL_NO_CONTEXT;
+  else
+#endif
+    egl_ctx = egl_display->egl_context;
+
   return egl_renderer->pf_eglCreateImage (egl_renderer->edpy,
-                                          egl_display->egl_context,
+                                          egl_ctx,
                                           target,
                                           buffer,
                                           attribs);
